# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-04-04 14:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aimmo', '0008_default_public_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='game',
            name='status',
            field=models.CharField(choices=[(b'r', b'running'), (b's', b'stopped'), (b'p', b'paused')], default=b'r', max_length=1),
        ),
    ]
